//
// wemu4 : F180 emulator for Windows ver4.00
//  (c) Junichi Akita (akita@fun.ac.jp), 2003.6.9
//

// wemu.cpp : AvP[VpNX̋@\`s܂B
//

#include "stdafx.h"
#include "wemu.h"
#include "MainFrm.h"
#include "wemuDoc.h"
#include "wemuView.h"
#include "SetParamDialog.h"

#include <afxdlgs.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// size of field
double FX = 274.0;  // width of field in [cm]
double FY = 152.5;  // height of field in [cm]
double GW = 50.0;   // width of goal in [cm]
double GH = 5.0;    // height of goal in [cm]

// physical dimensions of objects
double BR  = 2.0;  // radius of ball in [cm]
double FRX[5]; // X size of friend's robot
double FRY[5]; // Y size of friend's robot
double ERX[5]; // X size of enermy's robot
double ERY[5]; // Y size of enermy's robot
double DRWF[5];  // distance of wheels of friend robot
double DRWE[5];  // distance of wheels of enermy robot
double FMD = 5.0;  // distance of friend robot's marker from center
double FMr = 2.0;  // radius of friend robot's marker
double EMD = 5.0;  // distance of enermy robot's marker from center
double EMr = 2.0;  // radius of enermy robot's marker

// physical parameters of motion
double Kaf[5];  // acceleration coefficient of wheel velocity
double Kae[5];  // acceleration coefficient of wheel velocity
double EFR[5]; // reflection coefficient of friends' robots
double EER[5]; // reflection coefficient of enermys' robots
double EW = 0.99; // reflection coefficient of wall
double EB = 0.99; // reflection coefficient of ball
double GB = 0.90; // dumping coefficient of velocity of ball
double GFR[5]; // dumping coefficient of velocity of friend robot
double GER[5]; // dumping coefficient of velocity of enermy robot
double FMAX_SPEED[5]; // maximum speed of friend robots in [cm/s] 
double EMAX_SPEED[5]; // maximum speed of enermy robots in [cm/s] 

CString par_fname = "";
CString pos_fname = "";

Ball ball, ball0;
Robot frobot[5], frobot0[5];
Robot erobot[5], erobot0[5];
int NRF = 5, NRE = 5;

int devi0 = 1;
int f_score = 0, e_score = 0;
unsigned int t0 = 0;
double ts = 0.0, te = 0.0;
int start = 0;
double R;
int ox, oy;
Point rotate(Point p, double d)
{
	Point pp;
	pp.x = p.x * cos(d*PI180) - p.y * sin(d*PI180);
	pp.y = p.x * sin(d*PI180) + p.y * cos(d*PI180);
	return(pp);
}
CString fsHost = "localhost";
CString esHost = "localhost";
int cPort = 10030;
int fmPort = 10031;
int emPort = 10041;
int fdc;                  // tco\Pbgpt@CfBXNv^
int fdfm;                 // tco\Pbgpt@CfBXNv^
int fdem;                 // tco\Pbgpt@CfBXNv^
int selected = -1;

void init_condition()
{
	int i;
	ball0.x = FX/2; ball0.y = FY/2; ball0.vx = 0.0; ball0.vy = 0.0;	ball0.m = 100.0; ball0.tc = 0.0; ball0.tn = 0.001;
	ball0.ax = 0.0; ball0.ay = 0.0;
//	CString s; s.Format("%d %d %d %d",ball0.x,ball0.y,FX,FY); AfxMessageBox(s);

	frobot0[0].x = FX/2-20.0; frobot0[0].y = FY/2;   frobot0[0].dir = 0.0; frobot0[0].tc = 0.0; frobot0[0].tn = 0.001; frobot0[0].m = 1000.0; frobot0[0].use = 1; frobot0[0].vtL = 0.0; frobot0[0].vtR = 0.0; frobot0[0].vL = 0.0; frobot0[0].vR = 0.0; frobot0[0].vx = 0.0; frobot0[0].vy = 0.0; frobot0[0].ax = 0.0; frobot0[0].ay = 0.0;
	frobot0[1].x = FX/3;      frobot0[1].y = FY/2;   frobot0[1].dir = 0.0; frobot0[1].tc = 0.0; frobot0[1].tn = 0.001; frobot0[1].m = 1000.0; frobot0[1].use = 1; frobot0[1].vtL = 0.0; frobot0[1].vtR = 0.0; frobot0[1].vL = 0.0; frobot0[1].vR = 0.0; frobot0[1].vx = 0.0; frobot0[1].vy = 0.0; frobot0[1].ax = 0.0; frobot0[1].ay = 0.0;
	frobot0[2].x = FX/5;      frobot0[2].y = FY/3;   frobot0[2].dir = 0.0; frobot0[2].tc = 0.0; frobot0[2].tn = 0.001; frobot0[2].m = 1000.0; frobot0[2].use = 1; frobot0[2].vtL = 0.0; frobot0[2].vtR = 0.0; frobot0[2].vL = 0.0; frobot0[2].vR = 0.0; frobot0[2].vx = 0.0; frobot0[2].vy = 0.0; frobot0[2].ax = 0.0; frobot0[2].ay = 0.0;
	frobot0[3].x = FX/5;      frobot0[3].y = 2*FY/3; frobot0[3].dir = 0.0; frobot0[3].tc = 0.0; frobot0[3].tn = 0.001; frobot0[3].m = 1000.0; frobot0[3].use = 1; frobot0[3].vtL = 0.0; frobot0[3].vtR = 0.0; frobot0[3].vL = 0.0; frobot0[3].vR = 0.0; frobot0[3].vx = 0.0; frobot0[3].vy = 0.0; frobot0[3].ax = 0.0; frobot0[3].ay = 0.0;
	frobot0[4].x = 5+FRY[4]/2;   frobot0[4].y = FY/2;   frobot0[4].dir = 90.0; frobot0[4].tc = 0.0; frobot0[4].tn = 0.001; frobot0[4].m = 1000.0; frobot0[4].use = 1; frobot0[4].vtL = 0.0; frobot0[4].vtR = 0.0; frobot0[4].vL = 0.0; frobot0[4].vR = 0.0; frobot0[4].vx = 0.0; frobot0[4].vy = 0.0; frobot0[4].ax = 0.0; frobot0[4].ay = 0.0;

	erobot0[0].x = FX/2+20.0; erobot0[0].y = FY/2;   erobot0[0].dir = 180.0; erobot0[0].tc = 0.0; erobot0[0].tn = 0.001; erobot0[0].m = 1000.0; erobot0[0].use = 1; erobot0[0].vtL = 0.0; erobot0[0].vtR = 0.0; erobot0[0].vL = 0.0; erobot0[0].vR = 0.0; erobot0[0].vx = 0.0; erobot0[0].vy = 0.0; erobot0[0].ax = 0.0; erobot0[0].ay = 0.0;
	erobot0[1].x = 2*FX/3;    erobot0[1].y = FY/2;   erobot0[1].dir = 180.0; erobot0[1].tc = 0.0; erobot0[1].tn = 0.001; erobot0[1].m = 1000.0; erobot0[1].use = 1; erobot0[1].vtL = 0.0; erobot0[1].vtR = 0.0; erobot0[1].vL = 0.0; erobot0[1].vR = 0.0; erobot0[1].vx = 0.0; erobot0[1].vy = 0.0; erobot0[1].ax = 0.0; erobot0[1].ay = 0.0;
	erobot0[2].x = 4*FX/5;    erobot0[2].y = FY/3;   erobot0[2].dir = 180.0; erobot0[2].tc = 0.0; erobot0[2].tn = 0.001; erobot0[2].m = 1000.0; erobot0[2].use = 1; erobot0[2].vtL = 0.0; erobot0[2].vtR = 0.0; erobot0[2].vL = 0.0; erobot0[2].vR = 0.0; erobot0[2].vx = 0.0; erobot0[2].vy = 0.0; erobot0[2].ax = 0.0; erobot0[2].ay = 0.0;
	erobot0[3].x = 4*FX/5;    erobot0[3].y = 2*FY/3; erobot0[3].dir = 180.0; erobot0[3].tc = 0.0; erobot0[3].tn = 0.001; erobot0[3].m = 1000.0; erobot0[3].use = 1; erobot0[3].vtL = 0.0; erobot0[3].vtR = 0.0; erobot0[3].vL = 0.0; erobot0[3].vR = 0.0; erobot0[3].vx = 0.0; erobot0[3].vy = 0.0; erobot0[3].ax = 0.0; erobot0[3].ay = 0.0;
	erobot0[4].x = FX-5-FRY[4]/2;erobot0[4].y = FY/2;   erobot0[4].dir = 270.0; erobot0[4].tc = 0.0; erobot0[4].tn = 0.001; erobot0[4].m = 1000.0; erobot0[4].use = 1; erobot0[4].vtL = 0.0; erobot0[4].vtR = 0.0; erobot0[4].vL = 0.0; erobot0[4].vR = 0.0; erobot0[4].vx = 0.0; erobot0[4].vy = 0.0; erobot0[4].ax = 0.0; erobot0[4].ay = 0.0;


	ball.x = ball0.x; ball.y = ball0.y;
	ball.vx = ball0.vx; ball.vy = ball0.vy; ball.m = ball0.m; ball.ax = ball0.ax; ball.ay = ball0.ay;
	ball.tc = te; ball.tn = te+0.001;
	ball.tvc = te; ball.tvn = te+0.001;

	for (i = 0; i < 5; i++){
		frobot[i].x = frobot0[i].x; frobot[i].y = frobot0[i].y;
		frobot[i].vx = frobot0[i].vx; frobot[i].vy = frobot0[i].vy;
		frobot[i].ax = frobot0[i].ax; frobot[i].ay = frobot0[i].ay;
		frobot[i].dir = frobot0[i].dir;
		frobot[i].tc = te; frobot[i].tn = te+0.001;
		frobot[i].tvc = te; frobot[i].tvn = te+0.001;
		frobot[i].m = frobot0[i].m;
		frobot[i].vtL = frobot0[i].vtL; frobot[i].vtR = frobot0[i].vtR;
		frobot[i].vL = frobot0[i].vL; frobot[i].vR = frobot0[i].vR;

		erobot[i].x = erobot0[i].x; erobot[i].y = erobot0[i].y;
		erobot[i].vx = erobot0[i].vx; erobot[i].vy = erobot0[i].vy;
		erobot[i].ax = erobot0[i].ax; erobot[i].ay = erobot0[i].ay;
		erobot[i].dir = erobot0[i].dir;
		erobot[i].tc = te; erobot[i].tn = te+0.001;
		erobot[i].tvc = te; erobot[i].tvn = te+0.001;
		erobot[i].m = erobot0[i].m;
		erobot[i].vtL = erobot0[i].vtL; erobot[i].vtR = erobot0[i].vtR;
		erobot[i].vL = erobot0[i].vL; erobot[i].vR = erobot0[i].vR;
	}

	calc_robot_outline();

	for (i = 0; i < 5; i++){
		frobot[i].x0 = frobot[i].x;
		frobot[i].y0 = frobot[i].y;
		erobot[i].x0 = erobot[i].x;
		erobot[i].y0 = erobot[i].y;
	}
	for (i = 0; i < 5; i++){
		frobot[i].use = 1; erobot[i].use = 1;
	}
	NRF = 0; NRE = 0;
	for (i = 0; i < 5; i++){
		if (frobot[i].use == 1) NRF++;
		if (erobot[i].use == 1) NRE++;
	}
}


void init_param()
{
	int i;

	for (i = 0; i < 5; i++){
	  FRX[i] = 12.0; // X size of friend's robot
	  FRY[i] = 10.0; // Y size of friend's robot
	  ERX[i] = 12.0; // X size of enermy's robot
	  ERY[i] = 10.0; // Y size of enermy's robot
	  DRWF[i] = 8.0;  // distance of wheels of friend robot
	  DRWE[i] = 8.0;  // distance of wheels of enermy robot
	  Kaf[i] = 5.0;  // acceleration coefficient of wheel velocity
	  Kae[i] = 5.0;  // acceleration coefficient of wheel velocity
	  EFR[i] = 0.99; // reflection coefficient of friends' robots
	  EER[i] = 0.99; // reflection coefficient of enermys' robots
	  GFR[i] = 0.90; // dumping coefficient of velocity of friend robot
	  GER[i] = 0.90; // dumping coefficient of velocity of enermy robot
	  FMAX_SPEED[i] = 20.0; // maximum speed of friend robots in [cm/s] 
	  EMAX_SPEED[i] = 20.0; // maximum speed of enermy robots in [cm/s] 
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CWemuApp

BEGIN_MESSAGE_MAP(CWemuApp, CWinApp)
	//{{AFX_MSG_MAP(CWemuApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_APP_SET_PARAM, OnAppSetParam)
	ON_COMMAND(ID_APP_LOAD, OnAppLoad)
	ON_COMMAND(ID_APP_SAVE, OnAppSave)
	ON_COMMAND(ID_APP_LOAD_POS, OnAppLoadPos)
	ON_COMMAND(ID_APP_SAVE_POS, OnAppSavePos)
	//}}AFX_MSG_MAP
	// W̃t@C{hLg R}h
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWemuApp NX̍\z

CWemuApp::CWemuApp()
{
	// TODO: ̈ʒuɍ\zpR[hǉĂB
	//  InitInstance ̏dvȏׂċLqĂB
	init_param();
	init_condition();
}


/////////////////////////////////////////////////////////////////////////////
// B CWemuApp IuWFNg

CWemuApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CWemuApp NX̏

BOOL CWemuApp::InitInstance()
{
	// WIȏ
	// ̋@\gpAst@C̃TCY
	//  Έȉ̓̏[`̒sKvȂ
	//  폜ĂB
	if (!AfxSocketInit())
	{
		AfxMessageBox("Failed to AfxSocketInit()");
		return FALSE;
	}

#ifdef _AFXDLL
	Enable3dControls();			// L DLL ̒ MFC gpꍇɂ͂ĂяoĂB
#else
	Enable3dControlsStatic();	// MFC ƐÓIɃNĂꍇɂ͂ĂяoĂB
#endif

	// ݒ肪ۑ鉺̃WXg L[ύX܂B
	// Ж܂͏ȂǁAK؂ȕ
	// ύXĂB
	SetRegistryKey(_T("Local AppWizard-Generated Applications"));

	LoadStdProfileSettings(0);  // W INI t@C̃IvV[ނ܂ (MRU ܂)

	// AvP[Vp̃hLg ev[go^܂BhLg ev[g
	//  ̓hLgAt[ EBhEƃr[邽߂ɋ@\܂B

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CWemuDoc),
		RUNTIME_CLASS(CMainFrame),       // C SDI t[ EBhE
		RUNTIME_CLASS(CWemuView));
	AddDocTemplate(pDocTemplate);

	// DDEAfile open ȂǕW̃VF R}h̃R}hC͂܂B
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// R}hCŃfBXpb` R}hw肵܂B
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// C EBhEꂽ̂ŁA\ƍXVs܂B
	m_pMainWnd->SetWindowText("wemu4: F180 emulator for Windows ver 4.00");
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	struct sockaddr_in caddr;
    if ((fdc = socket(AF_INET, SOCK_DGRAM, 0)) < 0) AfxMessageBox("socket() error");
    memset((char *)&caddr, 0, sizeof(caddr));
    caddr.sin_family = AF_INET; caddr.sin_addr.s_addr = INADDR_ANY;
	caddr.sin_port = htons(cPort);
    if (bind(fdc, (struct sockaddr *)&caddr, sizeof(caddr)) < 0) AfxMessageBox("bind() error");

	struct sockaddr_in fmaddr;
    if ((fdfm = socket(AF_INET, SOCK_DGRAM, 0)) < 0) AfxMessageBox("socket() error");
    memset((char *)&fmaddr, 0, sizeof(fmaddr));
    fmaddr.sin_family = AF_INET; fmaddr.sin_addr.s_addr = INADDR_ANY;
	fmaddr.sin_port = htons(fmPort);
    if (bind(fdfm, (struct sockaddr *)&fmaddr, sizeof(fmaddr)) < 0) AfxMessageBox("bind() error");

	struct sockaddr_in emaddr;
    if ((fdem = socket(AF_INET, SOCK_DGRAM, 0)) < 0) AfxMessageBox("socket() error");
    memset((char *)&emaddr, 0, sizeof(emaddr));
    emaddr.sin_family = AF_INET; emaddr.sin_addr.s_addr = INADDR_ANY;
	emaddr.sin_port = htons(emPort);
    if (bind(fdem, (struct sockaddr *)&emaddr, sizeof(emaddr)) < 0) AfxMessageBox("bind() error");

	return TRUE;
}

int CWemuApp::ExitInstance() 
{
	// TODO: ̈ʒuɌŗL̏ǉ邩A܂͊{NXĂяoĂ
	closesocket(fdc); 
	closesocket(fdfm); 
	closesocket(fdem); 
	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// AvP[Ṽo[WŎg CAboutDlg _CAO

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// _CAO f[^
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard z֐̃I[o[Ch𐶐܂B
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // // DDX/DDV ̃T|[g
	//}}AFX_VIRTUAL

// Cve[V
protected:
	//{{AFX_MSG(CAboutDlg)
		// bZ[W nh͂܂B
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// bZ[W nh͂܂B
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// _CAOs邽߂̃AvP[V R}h
void CWemuApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CWemuApp R}h

void update_time_ball(double t)
{
	double v, dt0;
	v = sqrt(ball.vx*ball.vx + ball.vy*ball.vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = BR/(4.0*v);
	if (dt0 > 0.1) dt0 = 0.1;
	ball.tc = t; ball.tn = t + dt0;

}

void update_vtime_ball(double t)
{
	double a, v, dt0;
	a = sqrt(ball.ax*ball.ax + ball.ay*ball.ay);
	v = sqrt(ball.vx*ball.vx + ball.vy*ball.vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = v/(4.0*a);
	if (dt0 > 0.1) dt0 = 0.1;
	ball.tvc = t; ball.tvn = t + dt0;
}

void update_time_frobot(double t, int i)
{
	double v, dt0;
	v = sqrt(frobot[i].vx*frobot[i].vx + frobot[i].vy*frobot[i].vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = FRY[i]/(4.0*v);
	if (dt0 > 0.1) dt0 = 0.1;
	frobot[i].tc = t; frobot[i].tn = t + dt0;
}

void update_vtime_frobot(double t, int i)
{
	double a, v, dt0;
	a = sqrt(frobot[i].ax*frobot[i].ax + frobot[i].ay*frobot[i].ay);
	v = sqrt(frobot[i].vx*frobot[i].vx + frobot[i].vy*frobot[i].vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = v/(4.0*a);
	if (dt0 > 0.1) dt0 = 0.1;
	frobot[i].tvc = t; frobot[i].tvn = t + dt0;
}

void update_time_erobot(double t, int i)
{
	double v, dt0;
	v = sqrt(erobot[i].vx*erobot[i].vx + erobot[i].vy*erobot[i].vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = ERY[i]/(4.0*v);
	if (dt0 > 0.1) dt0 = 0.1;
	erobot[i].tc = t; erobot[i].tn = t + dt0;
}

void update_vtime_erobot(double t, int i)
{
	double a, v, dt0;
	a = sqrt(erobot[i].ax*erobot[i].ax + erobot[i].ay*erobot[i].ay);
	v = sqrt(erobot[i].vx*erobot[i].vx + erobot[i].vy*erobot[i].vy);
	if (v == 0.0) dt0 = 1.0; else dt0 = v/(4.0*a);
	if (dt0 > 0.1) dt0 = 0.1;
	erobot[i].tvc = t; erobot[i].tvn = t + dt0;
}

// solve a line f(p) throughing points pa & pb, and calcualte f(p0)*f(p1)
double cross(Point pa, Point pb, Point p0, Point p1)
{
	double a, b, c;
	double f0, f1;
	a = (pa.y - pb.y);
	b = (pb.x - pa.x);
	c = (pa.x*pb.y - pb.x*pa.y);
	f0 = a*p0.x + b*p0.y + c;
	f1 = a*p1.x + b*p1.y + c;
	return (f0*f1);
}

void update_position_ball(double t)
{
	double dt;
	dt = t - ball.tc;
	ball.x = ball.x + ball.vx * dt;
	ball.y = ball.y + ball.vy * dt;
}

void update_shape_frobot(int i){
	Point p, pp;
	p.x =  FRX[i]/2.0; p.y =  FRY[i]/2.0; pp = rotate(p, frobot[i].dir); frobot[i].p[0].x = pp.x + frobot[i].x; frobot[i].p[0].y = pp.y + frobot[i].y;
	p.x = -FRX[i]/2.0; p.y =  FRY[i]/2.0; pp = rotate(p, frobot[i].dir); frobot[i].p[1].x = pp.x + frobot[i].x; frobot[i].p[1].y = pp.y + frobot[i].y;
	p.x = -FRX[i]/2.0; p.y = -FRY[i]/2.0; pp = rotate(p, frobot[i].dir); frobot[i].p[2].x = pp.x + frobot[i].x; frobot[i].p[2].y = pp.y + frobot[i].y;
	p.x =  FRX[i]/2.0; p.y = -FRY[i]/2.0; pp = rotate(p, frobot[i].dir); frobot[i].p[3].x = pp.x + frobot[i].x; frobot[i].p[3].y = pp.y + frobot[i].y;
}

void update_position_frobot(double t, int i)
{
	double dt;
	dt = t - frobot[i].tc;
	frobot[i].x = frobot[i].x + frobot[i].vx * dt;
	frobot[i].y = frobot[i].y + frobot[i].vy * dt;
	update_shape_frobot(i);
}

void update_shape_erobot(int i)
{
	Point p, pp;
	p.x =  ERX[i]/2.0; p.y =  ERY[i]/2.0; pp = rotate(p, erobot[i].dir); erobot[i].p[0].x = pp.x + erobot[i].x; erobot[i].p[0].y = pp.y + erobot[i].y;
	p.x = -ERX[i]/2.0; p.y =  ERY[i]/2.0; pp = rotate(p, erobot[i].dir); erobot[i].p[1].x = pp.x + erobot[i].x; erobot[i].p[1].y = pp.y + erobot[i].y;
	p.x = -ERX[i]/2.0; p.y = -ERY[i]/2.0; pp = rotate(p, erobot[i].dir); erobot[i].p[2].x = pp.x + erobot[i].x; erobot[i].p[2].y = pp.y + erobot[i].y;
	p.x =  ERX[i]/2.0; p.y = -ERY[i]/2.0; pp = rotate(p, erobot[i].dir); erobot[i].p[3].x = pp.x + erobot[i].x; erobot[i].p[3].y = pp.y + erobot[i].y;
}

void update_position_erobot(double t, int i)
{
	double dt;
	dt = t - erobot[i].tc;
	erobot[i].x = erobot[i].x + erobot[i].vx * dt;
	erobot[i].y = erobot[i].y + erobot[i].vy * dt;
	update_shape_erobot(i);
}

BOOL CWemuApp::OnIdle(LONG lCount) 
{
	CTime theTime = CTime::GetCurrentTime();

	struct timeval tv;
	struct sockaddr_in caddr;
	fd_set fds;
	int ret, len;
	char buf[1024], tHostname[1024];
	int tPort;
	int flag;
	int i, j;
	CString send_string, ss;
	Point p, pp;

//				CString s;
//				s.Format("%.1f %.1f %.1f", frobot[0].x, frobot[0].y, frobot[0].dir);
//				((CFrameWnd *)AfxGetMainWnd())->SetMessageText(s);
//				Invalidate(NULL);

//	receive F's robot motion commands
	flag = 1;
	while(flag == 1){
		flag = 0;
		tv.tv_sec = 0; tv.tv_usec = 0;
		FD_ZERO(&fds);
		FD_SET(fdfm, &fds);
		if (select(10, &fds, (fd_set *)0, (fd_set *)0, &tv) < 0) AfxMessageBox("failed to select()");
		if(FD_ISSET(fdfm, &fds)){
			len = sizeof(caddr);
			flag = 1;
			if ((ret = recvfrom(fdfm, buf, 1023, 0, (struct sockaddr *)&caddr, &len)) < 0) AfxMessageBox("failed to recvfrom().");
			else{
				buf[ret] = '\0';
//				CString s;
//				s.Format("%s", buf);
//				((CFrameWnd *)AfxGetMainWnd())->SetMessageText(s);
//				Invalidate(NULL);
				int num;
				double vl, vr;
				num = atoi(strtok(buf, " \t\n"));
				vl = atof(strtok(NULL, " \t\n"));
				vr = atof(strtok(NULL, " \t\n"));
				// if keeper has 1 DOF:
//				if (num == 5) vr = vl;
				if (num > 0 && num < 6){
					frobot[num-1].vtL = FMAX_SPEED[num-1] * vl;
					frobot[num-1].vtR = FMAX_SPEED[num-1] * vr;
				}
			}
		}
	}

//	receive E's robot motion commands
	flag = 1;
	while(flag == 1){
		flag = 0;
		tv.tv_sec = 0; tv.tv_usec = 0;
		FD_ZERO(&fds);
		FD_SET(fdem, &fds);
		if (select(10, &fds, (fd_set *)0, (fd_set *)0, &tv) < 0) AfxMessageBox("failed to select()");
		if(FD_ISSET(fdem, &fds)){
			len = sizeof(caddr);
			flag = 1;
			if ((ret = recvfrom(fdem, buf, 1023, 0, (struct sockaddr *)&caddr, &len)) < 0) AfxMessageBox("failed to recvfrom().");
			else{
				buf[ret] = '\0';
				int num;
				double vl, vr;
				num = atoi(strtok(buf, " \t\n"));
				vl = atof(strtok(NULL, " \t\n"));
				vr = atof(strtok(NULL, " \t\n"));
				// if keeper has 1 DOF:
				//				if (num == 5) vr = vl;
				if (num > 0 && num < 6){
					erobot[num-1].vtL = EMAX_SPEED[num-1] * vl;
					erobot[num-1].vtR = EMAX_SPEED[num-1] * vr;
				}
			}
		}
	}


//client -> server
//[IPS:PORTF] or [IPS:PORTE]
//hostname port team robot#
//
//server -> client
//[hostname:port]
//t SX SY BX BY NRF NRE ....

// receive coordinate request
	flag = 1;
	while(flag == 1){
		flag = 0;
		tv.tv_sec = 0; tv.tv_usec = 0;
		FD_ZERO(&fds);
		FD_SET(fdc, &fds);
		if (select(10, &fds, (fd_set *)0, (fd_set *)0, &tv) < 0) AfxMessageBox("failed to select()");
		if(FD_ISSET(fdc, &fds)){
			len = sizeof(caddr);
			flag = 1;
			if ((ret = recvfrom(fdc, buf, 1023, 0, (struct sockaddr *)&caddr, &len)) < 0) AfxMessageBox("failed to recvfrom().");
			else{
				buf[ret] = '\0';
				// buf : received text (hostname port team robot#)

				strcpy(tHostname, strtok(buf, " \t\n"));
				tPort = atoi(strtok(NULL, " \t\n"));

				char team;
				int rnum;
				team = *strtok(NULL, " \t\n");
				if (team == 'y' || team == 'Y') team = 'Y';
				rnum = atoi(strtok(NULL, " \t\n")) - 1;
				int Nrf, Nre;
				Nrf=0;Nre=0;
				for(i=0;i<5;i++)if(frobot[i].use == 1) Nrf++;
				for(i=0;i<5;i++)if(erobot[i].use == 1) Nre++;
//				ss.Format("Y/target = %s : %d\n", tHostname, tPort);
//				((CFrameWnd *)AfxGetMainWnd())->SetMessageText(ss);
//				Invalidate(NULL);
				if (team == 'Y') send_string.Format("%d %d %d %d %d %d %d ", t0, MAP(FX), MAP(FY), MAP(ball.x0)+devi(), MAP(ball.y0)+devi(), Nrf, Nre);
				else send_string.Format("%d %d %d %d %d %d %d ", t0, MAP(FX), MAP(FY), MAP(ball.x0)+devi(), MAP(ball.y0)+devi(), Nre, Nrf);

//				ss.Format("%f %d / %f : %d",ball.x, ball.x0, R, MAP(ball.x));AfxMessageBox(ss); 
		
				if (team == 'Y'){
					if (frobot[rnum].use == 1){
						ss.Format("%d %d ", MAP(frobot[rnum].x0)+devi(), MAP(frobot[rnum].y0)+devi());
						send_string += ss;
						ss.Format("%d ", (int)(frobot[rnum].dir));
						send_string += ss;
					}
					else send_string += "-1 -1 0 ";
					
					for (i = 0; i < 5; i++){
						if (i != rnum){
							if (frobot[i].use == 1){
								ss.Format("%d %d ", MAP(frobot[i].x0)+devi(), MAP(frobot[i].y0)+devi());
								send_string += ss;
								ss.Format("%d ", (int)(frobot[i].dir));
								send_string += ss;
							}
						}
					}
					for (i = 0; i < 5; i++){
						if (erobot[i].use == 1){
							ss.Format("%d %d ", MAP(erobot[i].x0)+devi(), MAP(erobot[i].y0)+devi());
							send_string += ss;
							ss.Format("%d ", (int)(erobot[i].dir));
							send_string += ss;
						}
					}
				}
				else{
					if (erobot[rnum].use == 1){
						ss.Format("%d %d ", MAP(erobot[rnum].x0)+devi(), MAP(erobot[rnum].y0)+devi());
						send_string += ss;
						ss.Format("%d ", (int)(erobot[rnum].dir));
						send_string += ss;
					}
					else send_string += "-1 -1 0 ";
					
					for (i = 0; i < 5; i++){
						if (i != rnum){
							if (erobot[i].use == 1){
								ss.Format("%d %d ", MAP(erobot[i].x0)+devi(), MAP(erobot[i].y0)+devi());
								send_string += ss;
								ss.Format("%d ", (int)(erobot[i].dir));
								send_string += ss;
							}
						}
					}
					for (i = 0; i < 5; i++){
						if (frobot[i].use == 1){
							ss.Format("%d %d ", MAP(frobot[i].x0)+devi(), MAP(frobot[i].y0)+devi());
							send_string += ss;
							ss.Format("%d ", (int)(frobot[i].dir));
							send_string += ss;
						}
					}
				}
				send_string += "\n";

				pSocket = new CAsyncSocket;
				CWemuApp *app = (CWemuApp *)AfxGetApp();
				if (AfxSocketInit(NULL) == 0) AfxMessageBox("init error");
				if (pSocket->Create(0, SOCK_DGRAM) == 0) AfxMessageBox("Create() error");
				if (pSocket->Connect(tHostname, tPort) == 0) AfxMessageBox("Connect() error");
				if (app->pSocket->Send(send_string, strlen(send_string), 0) == SOCKET_ERROR) AfxMessageBox("Send() error");
				pSocket->Close();
			}
		}
	}

	double t;
	struct _timeb tstruct;
	_ftime( &tstruct );
	t = (double)tstruct.time + tstruct.millitm/1000.0 - ts; /* t :vOJn̎[ms] */

	double vx, vy, ax, ay, dt;

	if (start == 1){
		Point fr0[5], er0[5];
		for (i = 0; i < 5; i++){
			if (frobot[i].use == 1){
				fr0[i].x = frobot[i].x;
				fr0[i].y = frobot[i].y;
			}
			if (erobot[i].use == 1){
				er0[i].x = erobot[i].x;
				er0[i].y = erobot[i].y;
			}
		}
		if (t > ball.tvn){
			dt = t - ball.tvc;
			ball.ax = ball.vx * (GB - 1.0) / dt; ball.ay = ball.vy * (GB - 1.0) / dt;
			ball.vx *= GB; ball.vy *= GB;
			if (ball.y <= BR){
				ball.vy = -(EB*EW)*ball.vy; ball.y = BR;
			}
			if (ball.y >= FY-BR){
				ball.vy = -(EB*EW)*ball.vy; ball.y = FY-BR;
			}
			if (ball.x <= BR){
				if (ball.y <= (FY/2-GW/2) || ball.y >= (FY/2+GW/2)){
					ball.vx = -(EB*EW)*ball.vx; ball.x = BR;
				}
				else if (ball.x <= 0.0){
					e_score++;
					start = 0;
					struct _timeb tstruct;
					_ftime( &tstruct );
					te = (double)tstruct.time + tstruct.millitm/1000.0 - ts;
					init_condition();
					((CFrameWnd *)AfxGetMainWnd())->Invalidate();
				}
			}
			if (ball.x >= FX-BR){
				if (ball.y <= (FY/2-GW/2) || ball.y >= (FY/2+GW/2)){
					ball.vx = -(EB*EW)*ball.vx; ball.x = FX - BR;
				}
				else if (ball.x >= FX){
					f_score++;
					start = 0;
					struct _timeb tstruct;
					_ftime( &tstruct );
					te = (double)tstruct.time + tstruct.millitm/1000.0 - ts;
					init_condition();
					((CFrameWnd *)AfxGetMainWnd())->Invalidate();
				}
			}
			update_vtime_ball(t);
		}
		if (t > ball.tn) update_position_ball(t);

		for (i = 0; i < 5; i++){
			if (frobot[i].use == 1){
				if (t > frobot[i].tvn){
					double vx0, vy0;
					dt = t - frobot[i].tvc;
					vx0 = frobot[i].vx; vy0 = frobot[i].vy;
					frobot[i].vL += Kaf[i] * (frobot[i].vtL - frobot[i].vL) * dt; frobot[i].vR += Kaf[i] * (frobot[i].vtR - frobot[i].vR) * dt;

					p.x = frobot[i].vx; p.y = frobot[i].vy;
					pp = rotate(p, -frobot[i].dir);

					ax = -pp.y * (frobot[i].vR - frobot[i].vL) / DRWF[i]; ay =  pp.x * (frobot[i].vR - frobot[i].vL) / DRWF[i];
		
					vx = (frobot[i].vR + frobot[i].vL)/2.0 + ax * dt; vy = ay * dt;
					p.x = vx; p.y = vy;
					pp = rotate(p, frobot[i].dir);
	
					frobot[i].vx = pp.x; frobot[i].vy = pp.y;

					double r, dtx;
					dtx = t - frobot[i].tc;
					r = (frobot[i].vR - frobot[i].vL) * dtx;
					frobot[i].dir += (r / DRWF[i])/PI180;

					if (frobot[i].dir < 0.0) frobot[i].dir += 360.0;
					if (frobot[i].dir > 360.0) frobot[i].dir -= 360.0;

					frobot[i].ax = (vx0 - frobot[i].vx) / dt;
					frobot[i].ay = (vy0 - frobot[i].vy) / dt;
					update_vtime_frobot(t, i);
				}
				if (t > frobot[i].tn) update_position_frobot(t, i);
			}
			if (erobot[i].use == 1){
				if (t > erobot[i].tvn){
					double vx0, vy0;
					dt = t - erobot[i].tvc;
					vx0 = erobot[i].vx; vy0 = erobot[i].vy;
					erobot[i].vL += Kae[i] * (erobot[i].vtL - erobot[i].vL) * dt; erobot[i].vR += Kae[i] * (erobot[i].vtR - erobot[i].vR) * dt;

					p.x = erobot[i].vx; p.y = erobot[i].vy;
					pp = rotate(p, -erobot[i].dir);

					ax = -pp.y * (erobot[i].vR - erobot[i].vL) / DRWE[i]; ay =  pp.x * (erobot[i].vR - erobot[i].vL) / DRWE[i];
		
					vx = (erobot[i].vR + erobot[i].vL)/2.0 + ax * dt; vy = ay * dt;
					p.x = vx; p.y = vy;
					pp = rotate(p, erobot[i].dir);
	
					erobot[i].vx = pp.x; erobot[i].vy = pp.y;

					double r, dtx;
					dtx = t - erobot[i].tc;
					r = (erobot[i].vR - erobot[i].vL) * dtx;
					erobot[i].dir += (r / DRWE[i])/PI180;

					if (erobot[i].dir < 0.0) erobot[i].dir += 360.0;
					if (erobot[i].dir > 360.0) erobot[i].dir -= 360.0;

					erobot[i].ax = (vx0 - erobot[i].vx) / dt;
					erobot[i].ay = (vy0 - erobot[i].vy) / dt;
					update_vtime_erobot(t, i);
				}
				if (t > erobot[i].tn) update_position_erobot(t, i);
			}
		}

		Point pv[4], pr, pb;
		double f[4], fmin;
		int c1, c2;
		int j;
		//  calculation of collisions of ball with friend robots
		pb.x = ball.x; pb.y = ball.y;
		for (i = 0; i < 5; i++){
			if (frobot[i].use == 1){
				c1 = -1;
				pr.x = frobot[i].x; pr.y = frobot[i].y;
				p.x =  (FRX[i]/2.0+BR); p.y =  (FRY[i]/2.0+BR); pv[0] = rotate(p, frobot[i].dir); pv[0].x += frobot[i].x; pv[0].y += frobot[i].y;
				p.x = -(FRX[i]/2.0+BR); p.y =  (FRY[i]/2.0+BR); pv[1] = rotate(p, frobot[i].dir); pv[1].x += frobot[i].x; pv[1].y += frobot[i].y;
				p.x = -(FRX[i]/2.0+BR); p.y = -(FRY[i]/2.0+BR); pv[2] = rotate(p, frobot[i].dir); pv[2].x += frobot[i].x; pv[2].y += frobot[i].y;
				p.x =  (FRX[i]/2.0+BR); p.y = -(FRY[i] /2.0+BR); pv[3] = rotate(p, frobot[i].dir); pv[3].x += frobot[i].x; pv[3].y += frobot[i].y;
				for (j = 0; j < 4; j++) f[j] = cross(pv[j], pv[(j+1)%4], pb, pr);
				if (f[0] >= 0.0 && f[1] >= 0.0 && f[2] >= 0.0 && f[3] >= 0.0){
					fmin = 1.0e9;
					for (j = 0; j < 4; j++) if (f[j] < fmin) {fmin = f[j]; c1 = j;}
					c2 = (c1+1)%4;
					Point u1, u2;
					Point u1r, u2r;
					Point v1r, v2r;
					Point v1, v2;
					double m1, m2;
					double e;
					double theta;
					m1 = ball.m; m2 = frobot[i].m;
					e = EFR[i] * EB;
					u1.x = ball.vx; u1.y = ball.vy; u2.x = frobot[i].vx; u2.y = frobot[i].vy;
					theta = -frobot[i].dir - 90.0*(c1-2);
					u1r = rotate(u1, theta);
					u2r = rotate(u2, theta);

					v1r.x = u1r.x; v2r.x = u2r.x;
					v1r.y = ( (m1-e*m2)*u1r.y + (m2+e*m2)*u2r.y ) / (m1+m2);
					v2r.y = ( (m1+e*m1)*u1r.y + (m2-e*m1)*u2r.y ) / (m1+m2);

					v1 = rotate(v1r, -theta);
					v2 = rotate(v2r, -theta);
					ball.vx = v1.x; ball.vy = v1.y;
					frobot[i].vx = v2.x; frobot[i].vy = v2.y;
					update_vtime_ball(t);
					update_vtime_frobot(t, i);
					update_position_frobot(t, i);
					update_position_ball(t);
				}
			}
		}

		// calculation of collisions of ball with enermy robots
		pb.x = ball.x; pb.y = ball.y;
		for (i = 0; i < 5; i++){
			if (erobot[i].use == 1){
				c1 = -1;
				pr.x = erobot[i].x; pr.y = erobot[i].y;
				p.x =  (ERX[i]/2.0+BR); p.y =  (ERY[i]/2.0+BR); pv[0] = rotate(p, erobot[i].dir); pv[0].x += erobot[i].x; pv[0].y += erobot[i].y;
				p.x = -(ERX[i]/2.0+BR); p.y =  (ERY[i]/2.0+BR); pv[1] = rotate(p, erobot[i].dir); pv[1].x += erobot[i].x; pv[1].y += erobot[i].y;
				p.x = -(ERX[i]/2.0+BR); p.y = -(ERY[i]/2.0+BR); pv[2] = rotate(p, erobot[i].dir); pv[2].x += erobot[i].x; pv[2].y += erobot[i].y;
				p.x =  (ERX[i]/2.0+BR); p.y = -(ERY[i]/2.0+BR); pv[3] = rotate(p, erobot[i].dir); pv[3].x += erobot[i].x; pv[3].y += erobot[i].y;
				for (j = 0; j < 4; j++) f[j] = cross(pv[j], pv[(j+1)%4], pb, pr);
				if (f[0] >= 0.0 && f[1] >= 0.0 && f[2] >= 0.0 && f[3] >= 0.0){
					fmin = 1.0e9;
					for (j = 0; j < 4; j++) if (f[j] < fmin) {fmin = f[j]; c1 = j;}
					c2 = (c1+1)%4;
					Point u1, u2;
					Point u1r, u2r;
					Point v1r, v2r;
					Point v1, v2;
					double m1, m2;
					double e;
					double theta;
					m1 = ball.m; m2 = erobot[i].m;
					e = EER[i] * EB;
					u1.x = ball.vx; u1.y = ball.vy; u2.x = erobot[i].vx; u2.y = erobot[i].vy;
					theta = -erobot[i].dir - 90.0*(c1-2);
					u1r = rotate(u1, theta);
					u2r = rotate(u2, theta);

					v1r.x = u1r.x; v2r.x = u2r.x;
					v1r.y = ( (m1-e*m2)*u1r.y + (m2+e*m2)*u2r.y ) / (m1+m2);
					v2r.y = ( (m1+e*m1)*u1r.y + (m2-e*m1)*u2r.y ) / (m1+m2);

					v1 = rotate(v1r, -theta);
					v2 = rotate(v2r, -theta);
					ball.vx = v1.x; ball.vy = v1.y;
					erobot[i].vx = v2.x; erobot[i].vy = v2.y;
					update_vtime_ball(t);
					update_vtime_frobot(t, i);
					update_position_frobot(t, i);
					update_position_ball(t);
				}
			}
		}

		int k, l;
		// calculation of collisions of friend robot with friend robots
		for (i = 0; i < 5; i++){
			if (frobot[i].use == 1){
				for (k = 0; k < 4; k++){
					pb.x = frobot[i].p[k].x; pb.y = frobot[i].p[k].y;
					for (j = 0; j < 5; j++){
						if (i != j && frobot[j].use == 1){
							pr.x = frobot[j].x; pr.y = frobot[j].y;
							for (l = 0; l < 4; l++){
								pv[l].x = frobot[j].p[l].x; pv[l].y = frobot[j].p[l].y;
							}
							for (l = 0; l < 4; l++) f[l] = cross(pv[l], pv[(l+1)%4], pb, pr);
							if (f[0] >= 0.0 && f[1] >= 0.0 && f[2] >= 0.0 && f[3] >= 0.0){
							// -- collision of friend robot [i] with friend robot[j] was occured
								frobot[i].x = fr0[i].x;
								frobot[i].y = fr0[i].y;
								frobot[j].x = fr0[j].x;
								frobot[j].y = fr0[j].y;
							}
						}
					}
				}
			}
		}

		// calculation of collisions of enermy robot with enermy robots
		for (i = 0; i < 5; i++){
			if (erobot[i].use == 1){
				for (k = 0; k < 4; k++){
					pb.x = erobot[i].p[k].x; pb.y = erobot[i].p[k].y;
					for (j = 0; j < 5; j++){
						if (i != j && erobot[j].use == 1){
							pr.x = erobot[j].x; pr.y = erobot[j].y;
							for (l = 0; l < 4; l++){
								pv[l].x = erobot[j].p[l].x; pv[l].y = erobot[j].p[l].y;
							}
							for (l = 0; l < 4; l++) f[l] = cross(pv[l], pv[(l+1)%4], pb, pr);
							if (f[0] >= 0.0 && f[1] >= 0.0 && f[2] >= 0.0 && f[3] >= 0.0){
							// -- collision of enermy robot [i] with enermy robot[j] was occured
								erobot[i].x = er0[i].x;
								erobot[i].y = er0[i].y;
								erobot[j].x = er0[j].x;
								erobot[j].y = er0[j].y;
							}
						}
					}
				}
			}
		}

		// calculation of collisions of friend robot with enermy robots
		for (i = 0; i < 5; i++){
			if (erobot[i].use == 1){
				for (k = 0; k < 4; k++){
					pb.x = erobot[i].p[k].x; pb.y = erobot[i].p[k].y;
					for (j = 0; j < 5; j++){
						if (frobot[j].use == 1){
							pr.x = frobot[j].x; pr.y = frobot[j].y;
							for (l = 0; l < 4; l++){
								pv[l].x = frobot[j].p[l].x; pv[l].y = frobot[j].p[l].y;
							}
							for (l = 0; l < 4; l++) f[l] = cross(pv[l], pv[(l+1)%4], pb, pr);
							if (f[0] >= 0.0 && f[1] >= 0.0 && f[2] >= 0.0 && f[3] >= 0.0){
							// -- collision of enermy robot [i] with friend robot[j] was occured
								erobot[i].x = er0[i].x;
								erobot[i].y = er0[i].y;
								frobot[j].x = fr0[j].x;
								frobot[j].y = fr0[j].y;
							}
						}
					}
				}
			}
		}
		// calculation of collision of friend robot with wall
		double px, py;
		for (i = 0; i < 5; i++){
			for (j = 0; j < 4; j++){
				px = frobot[i].p[j].x; py = frobot[i].p[j].y;
				if (px <= 0.0 || px >= FX || py <= 0.0 || py >= FY){
					// -- collision of friend robot[i] with wall was occured
					frobot[i].x = fr0[i].x;
					frobot[i].y = fr0[i].y;
				}
			}
		}
		// calculation of collision of enermy robot with wall
		for (i = 0; i < 5; i++){
			for (j = 0; j < 4; j++){
				px = erobot[i].p[j].x; py = erobot[i].p[j].y;
				if (px <= 0.0 || px >= FX || py <= 0.0 || py >= FY){
					// -- collision of enermy robot[i] with wall was occured
					erobot[i].x = er0[i].x;
					erobot[i].y = er0[i].y;
				}
			}
		}

		if (t > ball.tn) update_time_ball(t);
		for (i = 0; i < 5; i++){
			if (frobot[i].use == 1) if (t > frobot[i].tn) update_time_frobot(t, i);
			if (erobot[i].use == 1) if (t > erobot[i].tn) update_time_erobot(t, i);
		}
	}
//	CString s;
//	s.Format("%d", t0);
//	((CFrameWnd *)AfxGetMainWnd())->SetMessageText(s);
//	((CFrameWnd *)AfxGetMainWnd())->PostMessage(WM_PAINT, 0, 0);
	//	return CWinApp::OnIdle(lCount);
	return(TRUE);
}

void CWemuApp::OnAppSetParam() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	CSetParamDialog dlg;
	dlg.DoModal();
	
}

void CWemuApp::OnAppLoad() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ

	int i;
	FILE *fp;
	CString fname;
	char buf[1024];
	if (par_fname == "") fname = "wemu4.par"; else fname = par_fname;
	CFileDialog dlg(TRUE, "par", fname, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "wemu4p[^(*.par) | *.par||");
	if (dlg.DoModal() == IDOK){
		par_fname = dlg.GetPathName();
		fp = fopen(par_fname, "r");

		fgets(buf, 1023, fp);
		FX = atof(strtok(buf, " \t\n"));
		FY = atof(strtok(NULL, " \t\n"));
		GW = atof(strtok(NULL, " \t\n"));
		GH = atof(strtok(NULL, " \t\n"));
		EW = atof(strtok(NULL, " \t\n"));

		fgets(buf, 1023, fp);
		BR = atof(strtok(buf, " \t\n"));
		EB = atof(strtok(NULL, " \t\n"));
		GB = atof(strtok(NULL, " \t\n"));

		for (i = 0; i < 5; i++){
			fgets(buf, 1023, fp);
			FRX[i] = atof(strtok(buf, " \t\n"));
			FRY[i] = atof(strtok(NULL, " \t\n"));
			DRWF[i] = atof(strtok(NULL, " \t\n"));
			Kaf[i] = atof(strtok(NULL, " \t\n"));
			EFR[i] = atof(strtok(NULL, " \t\n"));
			GFR[i] = atof(strtok(NULL, " \t\n"));
			FMAX_SPEED[i] = atof(strtok(NULL, " \t\n"));
		}
		for (i = 0; i < 5; i++){
			fgets(buf, 1023, fp);
			ERX[i] = atof(strtok(buf, " \t\n"));
			ERY[i] = atof(strtok(NULL, " \t\n"));
			DRWE[i] = atof(strtok(NULL, " \t\n"));
			Kae[i] = atof(strtok(NULL, " \t\n"));
			EER[i] = atof(strtok(NULL, " \t\n"));
			GER[i] = atof(strtok(NULL, " \t\n"));
			EMAX_SPEED[i] = atof(strtok(NULL, " \t\n"));
		}
		fclose(fp);
		calc_robot_outline();
		((CFrameWnd *)AfxGetMainWnd())->Invalidate();
	}
}

void CWemuApp::OnAppSave() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	int i;
	CString fname;
	FILE *fp;

	if (par_fname == "") fname = "wemu4.par"; else fname = par_fname;
	CFileDialog dlg(FALSE, "par", fname, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "wemu4p[^(*.par) | *.par||");
	if (dlg.DoModal() == IDOK){
		fp = fopen(dlg.GetPathName(), "w");
		fprintf(fp, "%f %f %f %f %f\n", FX, FY, GW, GH, EW);
		fprintf(fp, "%f %f %f\n", BR, EB, GB);
		for (i = 0; i < 5; i++){
			fprintf(fp, "%f %f %f %f %f %f %f\n", FRX[i], FRY[i], DRWF[i], Kaf[i], EFR[i], GFR[i], FMAX_SPEED[i]);
		}
		for (i = 0; i < 5; i++){
			fprintf(fp, "%f %f %f %f %f %f %f\n", ERX[i], ERY[i], DRWE[i], Kae[i], EER[i], GER[i], EMAX_SPEED[i]);
		}
		fclose(fp);	
	}
}

void CWemuApp::OnAppLoadPos() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	int i;
	FILE *fp;
	CString fname;
	char buf[1024];
	if (pos_fname == "") fname = "wemu4.pos"; else fname = pos_fname;
	CFileDialog dlg(TRUE, "pos", fname, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "wemu4ʒu(*.pos) | *.pos||");
	if (dlg.DoModal() == IDOK){
		pos_fname = dlg.GetPathName();
		fp = fopen(pos_fname, "r");

		fgets(buf, 1023, fp);
		ball.x = atof(strtok(buf, " \t\n"));
		ball.y = atof(strtok(NULL, " \t\n"));
		for (i = 0; i < 5; i++){
			fgets(buf, 1023, fp);
			frobot[i].use = atoi(strtok(buf, " \t\n"));
			frobot[i].x = atof(strtok(NULL, " \t\n"));
			frobot[i].y = atof(strtok(NULL, " \t\n"));
			frobot[i].dir = atof(strtok(NULL, " \t\n"));
		}
		for (i = 0; i < 5; i++){
			fgets(buf, 1023, fp);
			erobot[i].use = atoi(strtok(buf, " \t\n"));
			erobot[i].x = atof(strtok(NULL, " \t\n"));
			erobot[i].y = atof(strtok(NULL, " \t\n"));
			erobot[i].dir = atof(strtok(NULL, " \t\n"));
		}
		fclose(fp);
		calc_robot_outline();
		((CFrameWnd *)AfxGetMainWnd())->Invalidate();
	}
	
}

void CWemuApp::OnAppSavePos() 
{
	// TODO: ̈ʒuɃR}h nhp̃R[hǉĂ
	int i;
	CString fname;
	FILE *fp;

	if (pos_fname == "") fname = "wemu4.pos"; else fname = pos_fname;
	CFileDialog dlg(FALSE, "pos", fname, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "wemu4ʒu(*.pos) | *.pos||");
	if (dlg.DoModal() == IDOK){
		fp = fopen(dlg.GetPathName(), "w");
		fprintf(fp, "%f %f\n", ball.x, ball.y);
		for (i = 0; i < 5; i++){
			fprintf(fp, "%d %f %f %f\n", frobot[i].use, frobot[i].x, frobot[i].y, frobot[i].dir);
		}
		for (i = 0; i < 5; i++){
			fprintf(fp, "%d %f %f %f\n", erobot[i].use, erobot[i].x, erobot[i].y, erobot[i].dir);
		}
		fclose(fp);	
	}
}
