//
// wemu4 : F180 emulator for Windows ver4.00
//  (c) Junichi Akita (akita@fun.ac.jp), 2003.6.9
//

// wemu.h : WEMU AvP[ṼC wb_[ t@C
//

#if !defined(AFX_WEMU_H__4B78EBE5_ACCF_11D3_AFFB_BA604A3F592B__INCLUDED_)
#define AFX_WEMU_H__4B78EBE5_ACCF_11D3_AFFB_BA604A3F592B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // C V{
#include <winsock.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <math.h>
#include <stdlib.h>

extern void init_condition();

#define PI180 (3.14159/180.0)

#define FRAME_CYCLE (33) // frame cycle in [msec]

// definition of field
extern double FX;  // width of field in [cm]
extern double FY;  // height of field in [cm]
extern double GW;  // width of goal in [cm]
extern double GH;  // height of goal in [cm]

extern CString par_fname;
extern CString pos_fname;

// definition of physical dimensions of objects
extern double BR;  // radius of ball in [cm]
extern double FRX[5]; // X size of friend's robot
extern double FRY[5]; // Y size of friend's robot
extern double ERX[5]; // X size of enermy's robot
extern double ERY[5]; // Y size of enermy's robot
extern double DRWF[5];  // distance of wheels of friend robot
extern double DRWE[5];  // distance of wheels of enermy robot
extern double FMD; // distance of friend robot's marker from center
extern double FMr; // radius of friend robot's marker
extern double EMD; // distance of enermy robot's marker from center
extern double EMr; // radius of enermy robot's marker

// definition of physical parameters of motion
extern double Kaf[5];  // acceleration coefficient of wheel velocity
extern double Kae[5];  // acceleration coefficient of wheel velocity
extern double EFR[5];  // reflection coefficient of friends' robots
extern double EER[5];  // reflection coefficient of enermys' robots
extern double EW;   // reflection coefficient of wall
extern double EB;   // reflection coefficient of ball
extern double GB;   // dumping coefficient of velocity of ball
extern double GFR[5];  // dumping coefficient of velocity of friend robot
extern double GER[5];  // dumping coefficient of velocity of enermy robot
extern double FMAX_SPEED[5]; // maximum speed of friend robots in [cm/s] 
extern double EMAX_SPEED[5]; // maximum speed of enermy robots in [cm/s] 

// definition of colors
#define YPING RGB(200,200,0)
#define FIELD RGB(0,60,0)
#define BPING RGB(0,200,255)
#define BALL  RGB(255,0,0)
#define YMARK  RGB(0,200,0)
#define BMARK  RGB(200,0,100)
#define LINE  RGB(255,255,255)

typedef struct{double x,y;} Point;
extern Point rotate(Point p, double d);

typedef struct{
	double x, y;
	double x0, y0;
	double vx, vy;
	double ax, ay;
	double m;
	double tc, tn;
	double tvc, tvn;
} Ball;

typedef struct{
	double x, y;
	double x0, y0;
	double vx, vy;
	double m;
	double dir;
	double tc, tn;
	int use;
	double vL, vR;
	double vtL, vtR;
	Point p[4];
	double ax, ay;
	double tvc, tvn;
} Robot;

extern Ball ball, ball0;
extern Robot frobot[5], frobot0[5];
extern Robot erobot[5], erobot0[5];
extern int NRF, NRE;

extern int f_score, e_score;
extern int devi0;
extern unsigned int t0;
extern double ts, te;
extern int selected;

extern int start;
extern double R;
extern int ox, oy;
extern int cPort, fmPort, emPort;
extern int fdc;
extern int fdfm, fdem;
extern void init_condition();
extern void calc_robot_outline();

#define MAP(x) ((int)(R*(x)))
#define RAND255() (rand() & 0xff)
#define devi() ((int)((((double)RAND255()/63.25)-2.0) * (double)devi0))

/////////////////////////////////////////////////////////////////////////////
// CWemuApp:
// ̃NX̓̒`ɊւĂ wemu.cpp t@CQƂĂB
//

class CWemuApp : public CWinApp
{
public:
	CWemuApp();
	CAsyncSocket *pSocket;

// I[o[Ch
	// ClassWizard ͉z֐̃I[o[Ch𐶐܂B
	//{{AFX_VIRTUAL(CWemuApp)
	public:
	virtual BOOL InitInstance();
	virtual BOOL OnIdle(LONG lCount);
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Cve[V

	//{{AFX_MSG(CWemuApp)
	afx_msg void OnAppAbout();
	afx_msg void OnAppSetParam();
	afx_msg void OnAppLoad();
	afx_msg void OnAppSave();
	afx_msg void OnAppLoadPos();
	afx_msg void OnAppSavePos();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio ͑Os̒Oɒǉ̐錾}܂B

#endif // !defined(AFX_WEMU_H__4B78EBE5_ACCF_11D3_AFFB_BA604A3F592B__INCLUDED_)
